const fs = require('fs')
const path = require('path')

function contains(arr, val) {
  return arr && arr.indexOf(val) !== -1
}

const atPrefix = new RegExp('^@', 'g')
function readDir(dirName) {
  if (!fs.existsSync(dirName)) {
    return []
  }

  try {
    return fs
      .readdirSync(dirName)
      .map(function(module) {
        if (atPrefix.test(module)) {
          // reset regexp
          atPrefix.lastIndex = 0
          try {
            return fs
              .readdirSync(path.join(dirName, module))
              .map(function(scopedMod) {
                return module + '/' + scopedMod
              })
          } catch (e) {
            return [module]
          }
        }
        return module
      })
      .reduce(function(prev, next) {
        return prev.concat(next)
      }, [])
  } catch (e) {
    return []
  }
}

function readFromPackageJson(options) {
  if (typeof options !== 'object') {
    options = {}
  }
  // read the file
  let packageJson
  try {
    const fileName = options.fileName || 'package.json'
    const packageJsonString = fs.readFileSync(
      path.join(process.cwd(), './' + fileName),
      'utf8'
    )
    packageJson = JSON.parse(packageJsonString)
  } catch (e) {
    return []
  }
  // sections to search in package.json
  let sections = [
    'dependencies',
    'devDependencies',
    'peerDependencies',
    'optionalDependencies',
  ]
  if (options.include) {
    sections = [].concat(options.include)
  }
  if (options.exclude) {
    sections = sections.filter(function(section) {
      return [].concat(options.exclude).indexOf(section) === -1
    })
  }
  // collect dependencies
  const deps = {}
  sections.forEach(function(section) {
    Object.keys(packageJson[section] || {}).forEach(function(dep) {
      deps[dep] = true
    })
  })
  return Object.keys(deps)
}

function containsPattern(arr, val) {
  return (
    arr &&
    arr.some(function(pattern) {
      if (pattern instanceof RegExp) {
        return pattern.test(val)
      } else if (typeof pattern === 'function') {
        return pattern(val)
      } else {
        return pattern == val
      }
    })
  )
}

module.exports = {
  contains,
  readDir,
  readFromPackageJson,
  containsPattern,
}
